package info.textgrid.lab.collatex.ui.commands;

import info.textgrid.lab.collatex.ui.parts.CollationSetEditor;
import info.textgrid.lab.collatex.ui.parts.RegularisationEditor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.EditorAreaHelper;
import org.eclipse.ui.internal.EditorPane;
import org.eclipse.ui.internal.EditorSashContainer;
import org.eclipse.ui.internal.EditorStack;
import org.eclipse.ui.internal.PartSite;
import org.eclipse.ui.internal.WorkbenchPage;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;

/**
 * This is a helper that arranges the CollateX editors. Typically, when users
 * have opened both CollationSet and Regularisation editors, both should be
 * shown side-by-side.
 * 
 * Unfortunately, there is no real API for this, so we'll have to use internal
 * stuff a lot.
 * 
 * 
 * 
 * parts of this come from or are inspired by Dimitro Missoh, cf.
 * http://eclipse.dzone.com/tips/programmatically-split-editor-
 * 
 * @author tv
 * 
 */
@SuppressWarnings("restriction")
// splitting editors doesn't seem to be API
public class ArrangeEditorsHandler extends AbstractHandler implements IHandler {

	/*
	 * EdititorArea & co
	 * 
	 * The EditorArea is of class EditorSashContainer
	 */

	/**
	 * Returns true iff the given workbook does contain at least one editor with
	 * the given ID.
	 */
	private static boolean containsEditor(final EditorStack workbook,
			final String editorID) {
		for (final EditorPane editorPane : workbook.getEditors())
			if (editorPane.getID().equals(editorID))
				return true;
		return false;
	}

	public static void sortItOut() {
		final Function<Object, EditorStack> toEditorStack = new Function<Object, EditorStack>() {

			@Override
			public EditorStack apply(final Object input) {
				return (EditorStack) input;
			}

		};

		final IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		final EditorAreaHelper editorAreaHelper = ((WorkbenchPage) page)
				.getEditorPresentation();

		// First, identify our stacks if they exist
		EditorStack cseStack = null;
		EditorStack regStack = null;
		final ArrayList<?> workbooks = editorAreaHelper.getWorkbooks();
		for (final EditorStack stack : Iterables.transform(workbooks,
				toEditorStack))
			if (cseStack == null
			&& containsEditor(stack, CollationSetEditor.ID))
				cseStack = stack;
			else if (regStack == null
					&& containsEditor(stack, RegularisationEditor.ID))
				regStack = stack;

		if (cseStack == null && regStack == null && workbooks.size() > 0) {
			cseStack = (EditorStack) workbooks.get(0);
			if (workbooks.size() > 1)
				regStack = (EditorStack) workbooks.get(1);
		}

		// Find relevant editors
		final List<IEditorPart> cses = Lists.newArrayList();
		final List<IEditorPart> regs = Lists.newArrayList();
		final IEditorReference[] editorReferences = page.getEditorReferences();
		for (final IEditorReference ref : editorReferences)
			if (ref.getId().equals(CollationSetEditor.ID))
				cses.add(ref.getEditor(true));
			else if (ref.getId().equals(RegularisationEditor.ID))
				regs.add(ref.getEditor(true));

		final EditorSashContainer editorArea = (EditorSashContainer) editorAreaHelper
				.getLayoutPart();

		for (final IEditorPart editor : cses) {
			final EditorPane pane = (EditorPane) ((PartSite) editor.getSite())
					.getPane();
			final EditorStack workbook = (EditorStack) pane.getContainer();
			if (workbook != cseStack)
				if (cseStack == null) {
					cseStack = EditorStack.newEditorWorkbook(editorArea,
							(WorkbenchPage) page);
					editorArea.stack(pane, cseStack);
					cseStack.setContainer(editorArea);
					editorArea.add(cseStack);
				} else
					editorArea.stack(pane, cseStack);
		}

		for (final IEditorPart editor : regs) {
			final EditorPane pane = (EditorPane) ((PartSite) editor.getSite())
					.getPane();
			final EditorStack workbook = (EditorStack) pane.getContainer();
			if (workbook != regStack)
				if (regStack == null) {
					regStack = EditorStack.newEditorWorkbook(editorArea,
							(WorkbenchPage) page);
					editorArea.stack(pane, regStack);
					regStack.setContainer(editorArea);
					editorArea.add(regStack);
				} else
					editorArea.stack(pane, regStack);
		}


	}



	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {
		sortItOut();
		return null;
	}

}
